﻿// This is an editor for the original DOS version of Family Feud.
// Created By: Dean Tersigni.
// Version: 2020-12-08.
// For latest version, check: http://www.thealmightyguru.com/Games/Games-Hacking.html

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace FamilyFeudEditor
{
	public partial class frm_FamilyFeudEditor : Form
	{
		// Setup Structs for all data structures.
		public struct FFAnswer
		{
			public string answer;
			public byte percentage;
		}

		public struct FFQuestion
		{
			public byte prefix;
			public string question;
			public FFAnswer[] answers;
		}

		public struct FFFile
		{
			public FFQuestion[] questions;
		}

		public FFFile[] Files;
		public bool filesLoaded = false;
		public bool Debug = false;
		public string Folder = Directory.GetCurrentDirectory();

		public int Round;
		public int QuestionBank;
		public int QuestionNo;

		public frm_FamilyFeudEditor()
		{
			InitializeComponent();

			// Create an array to house for all the questions and answers.
			Files = new FFFile[76];
			for (int j = 0; j < 76; j++)
			{
				Files[j].questions = new FFQuestion[10];
				
				for (int i = 0; i < 10; i++)
				{
					Files[j].questions[i].answers = new FFAnswer[12];
				}
			}
		}

		private void frm_FamilyFeudEditor_Load(object sender, EventArgs e)
		{
			// Check for override debug folder.
			if (Debug)
				Folder = @"H:\DOS\Feud\";

			// Hide the Revert button unless a backup exists.
			if (!Directory.Exists(Path.Combine(Folder, "Original")))
				but_Restore.Visible = false;

			// Load the questions from the files.
			LoadQuestions();

			// Set up the default form values.
			cbo_Round.Items.Add("Face-Off Questions");
			cbo_Round.Items.Add("Fast Money Questions");
			cbo_Round.SelectedIndex = 0;
			LoadQuestionBankCombo();
			cbo_Prefix.Items.Add("0 - No Prefix");
			cbo_Prefix.Items.Add("1 - Name the...");
			cbo_Prefix.Items.Add("2 - Name Something...");
			cbo_Prefix.Items.Add("3 - Name a...");
			cbo_Prefix.Items.Add("4 - Name an...");
			cbo_Prefix.Items.Add("5 - Tell me...");
			txt_QuestionNo.Text = "1";
			CheckQuestionButtons();
			UpdateForm();
		}

		private void but_Previous_Click(object sender, EventArgs e)
		{
			int questionNo = Convert.ToInt32(txt_QuestionNo.Text);
			if (questionNo > 1)
			{
				SaveQuestion();
				questionNo--;
				txt_QuestionNo.Text = questionNo.ToString();
				CheckQuestionButtons();
				UpdateForm();
			}
		}

		private void but_Next_Click(object sender, EventArgs e)
		{
			int questionNo = Convert.ToInt32(txt_QuestionNo.Text);
			if (questionNo < 10)
			{
				SaveQuestion();
				questionNo++;
				txt_QuestionNo.Text = questionNo.ToString();
				CheckQuestionButtons();
				UpdateForm();
			}
		}

		private void but_Close_Click(object sender, EventArgs e)
		{
			this.Close();
		}

		private void frm_FamilyFeudEditor_FormClosing(object sender, FormClosingEventArgs e)
		{
			if (filesLoaded == true)
			{
				DialogResult window = MessageBox.Show(
					"Are you sure you want to close? Any unsaved changes will be lost!",
					"Confirm Close",
					MessageBoxButtons.YesNo,
					MessageBoxIcon.Warning);

				e.Cancel = (window == DialogResult.No);
			}
		}

		public void LoadQuestions()
		{
			// Load all the questions off disk.
			for (int fileNo = 0; fileNo < 76; fileNo++)
			{
				// Determine the file name.
				string fileName;
				if (fileNo < 46)
					fileName = "qs" + (fileNo + 1).ToString() + ".ff";
				else
					fileName = "fq" + (fileNo - 45).ToString() + ".ff";

				fileName = Path.Combine(Folder, fileName);

				// Check to see if the file exists in the current location.
				if (!File.Exists(fileName))
				{
					MessageBox.Show(
						"Cannot find file: " + fileName + "!",
						"Missing File",
						MessageBoxButtons.OK,
						MessageBoxIcon.Error);

					this.Close();
					return;
				}

				// Load the Family Feud quetions and answers.
				using (BinaryReader bw = new BinaryReader(File.Open(fileName, FileMode.Open)))
				{
					for(int questionNo = 0; questionNo < 10; questionNo++)
					{ 
						// Read the question.
						string question = "";

						byte readByte;
						do
						{
							readByte = bw.ReadByte();
							question += ((char)readByte).ToString();
						} while (readByte != 10);

						question = question.Substring(0, question.Length - 2);      // Remove the CRLF.
						Files[fileNo].questions[questionNo].question = Decode(question);

						byte prefix = bw.ReadByte();
						prefix -= 33;

						Files[fileNo].questions[questionNo].prefix = prefix;

						byte totalAnswers = bw.ReadByte();
						totalAnswers -= 33;

						// Read all the answers.
						for (int answerNo = 0; answerNo < totalAnswers; answerNo++)
						{
							// First byte is the percentage.
							byte percentage = bw.ReadByte();
							percentage -= 33;

							Files[fileNo].questions[questionNo].answers[answerNo].percentage = percentage;

							// Remaining is the answer.
							string answer = "";
							do
							{
								readByte = bw.ReadByte();
								answer += ((char)readByte).ToString();
							} while (readByte != 10);

							answer = answer.Substring(0, answer.Length - 2);      // Remove the CRLF.
							Files[fileNo].questions[questionNo].answers[answerNo].answer = Decode(answer);
						}
					}
				}

				filesLoaded = true;
			}
		}

		public string Decode(string encodedText)
		{
			string decodedText = "";

			for (int i = 0; i < encodedText.Length; i++)
			{
				byte character = (byte)encodedText[i];

				// All characters are rotated back 33.
				character -= 33;

				decodedText += ((char)character).ToString();
			}

			return decodedText;
		}

		// Encodes a string to the Family Feud encryption format.
		public byte[] Encode(string plainText)
		{
			byte[] encodedData = new byte[plainText.Length];

			for (int i = 0; i < plainText.Length; i++)
			{
				byte character = (byte)plainText[i];

				// All characters are rotated forward by 33.
				character += 33;
				encodedData[i] = character;
			}

			return encodedData;
		}

		public void UpdateForm()
		{
			// Determine the question offset based on the form's specified values.
			int round = cbo_Round.SelectedIndex;
			int file = cbo_QuestionBank.SelectedIndex;
			if (round == 1)
				file += 46;
			int question = (Convert.ToInt32(txt_QuestionNo.Text) - 1);

			// Update the form with the specified question.
			cbo_Prefix.SelectedIndex = Files[file].questions[question].prefix;
			txt_Question.Text = Files[file].questions[question].question;

			txt_Answer1.Text = Files[file].questions[question].answers[0].answer;
			txt_Percentage1.Text = Files[file].questions[question].answers[0].percentage.ToString();
			txt_Answer2.Text = Files[file].questions[question].answers[1].answer;
			txt_Percentage2.Text = Files[file].questions[question].answers[1].percentage.ToString();
			txt_Answer3.Text = Files[file].questions[question].answers[2].answer;
			txt_Percentage3.Text = Files[file].questions[question].answers[2].percentage.ToString();
			txt_Answer4.Text = Files[file].questions[question].answers[3].answer;
			txt_Percentage4.Text = Files[file].questions[question].answers[3].percentage.ToString();
			txt_Answer5.Text = Files[file].questions[question].answers[4].answer;
			txt_Percentage5.Text = Files[file].questions[question].answers[4].percentage.ToString();
			txt_Answer6.Text = Files[file].questions[question].answers[5].answer;
			txt_Percentage6.Text = Files[file].questions[question].answers[5].percentage.ToString();
			txt_Answer7.Text = Files[file].questions[question].answers[6].answer;
			txt_Percentage7.Text = Files[file].questions[question].answers[6].percentage.ToString();
			txt_Answer8.Text = Files[file].questions[question].answers[7].answer;
			txt_Percentage8.Text = Files[file].questions[question].answers[7].percentage.ToString();
			txt_Answer9.Text = Files[file].questions[question].answers[8].answer;
			txt_Percentage9.Text = Files[file].questions[question].answers[8].percentage.ToString();
			txt_Answer10.Text = Files[file].questions[question].answers[9].answer;
			txt_Percentage10.Text = Files[file].questions[question].answers[9].percentage.ToString();
			txt_Answer11.Text = Files[file].questions[question].answers[10].answer;
			txt_Percentage11.Text = Files[file].questions[question].answers[10].percentage.ToString();
			txt_Answer12.Text = Files[file].questions[question].answers[11].answer;
			txt_Percentage12.Text = Files[file].questions[question].answers[11].percentage.ToString();

			// Update the memory variables with the new form values.
			Round = cbo_Round.SelectedIndex;
			QuestionBank = cbo_QuestionBank.SelectedIndex;
			QuestionNo = Convert.ToInt32(txt_QuestionNo.Text) - 1;
		}

		public void SaveQuestion()
		{
			// Save using the old memory values before they were changed by the user.
			int round = Round;
			int file = QuestionBank;
			if (round == 1)
				file += 46;
			int question = QuestionNo;

			// Update the memory with the form.
			Files[file].questions[question].question = txt_Question.Text;
			Files[file].questions[question].prefix = (byte)cbo_Prefix.SelectedIndex;

			Files[file].questions[question].answers[0].answer = txt_Answer1.Text;
			Files[file].questions[question].answers[0].percentage = VerifyPercentage(txt_Percentage1.Text);
			Files[file].questions[question].answers[1].answer = txt_Answer2.Text;
			Files[file].questions[question].answers[1].percentage = VerifyPercentage(txt_Percentage2.Text);
			Files[file].questions[question].answers[2].answer = txt_Answer3.Text;
			Files[file].questions[question].answers[2].percentage = VerifyPercentage(txt_Percentage3.Text);
			Files[file].questions[question].answers[3].answer = txt_Answer4.Text;
			Files[file].questions[question].answers[3].percentage = VerifyPercentage(txt_Percentage4.Text);
			Files[file].questions[question].answers[4].answer = txt_Answer5.Text;
			Files[file].questions[question].answers[4].percentage = VerifyPercentage(txt_Percentage5.Text);
			Files[file].questions[question].answers[5].answer = txt_Answer6.Text;
			Files[file].questions[question].answers[5].percentage = VerifyPercentage(txt_Percentage6.Text);
			Files[file].questions[question].answers[6].answer = txt_Answer7.Text;
			Files[file].questions[question].answers[6].percentage = VerifyPercentage(txt_Percentage7.Text);
			Files[file].questions[question].answers[7].answer = txt_Answer8.Text;
			Files[file].questions[question].answers[7].percentage = VerifyPercentage(txt_Percentage8.Text);
			Files[file].questions[question].answers[8].answer = txt_Answer9.Text;
			Files[file].questions[question].answers[8].percentage = VerifyPercentage(txt_Percentage9.Text);
			Files[file].questions[question].answers[9].answer = txt_Answer10.Text;
			Files[file].questions[question].answers[9].percentage = VerifyPercentage(txt_Percentage10.Text);
			Files[file].questions[question].answers[10].answer = txt_Answer11.Text;
			Files[file].questions[question].answers[10].percentage = VerifyPercentage(txt_Percentage11.Text);
			Files[file].questions[question].answers[11].answer = txt_Answer12.Text;
			Files[file].questions[question].answers[11].percentage = VerifyPercentage(txt_Percentage12.Text);
		}

		// Verifys that the percentage string passed in is a number between 0-100 and returns its byte value.
		public byte VerifyPercentage(string input)
		{
			byte number = 0;

			try
			{
				int test = Convert.ToInt32(input);
				if(test > 0 && test < 101)
				{
					number = (byte)test;
				}
			}
			catch {}

			return number;
		}

		private void but_Save_Click(object sender, EventArgs e)
		{
			// Overwrite the files on disk with the questions in memory.

			// First, save the question we're on.
			SaveQuestion();

			// Backup the original game questions.
			if (!Directory.Exists(Path.Combine(Folder, "Original")))
			{
				Directory.CreateDirectory(Path.Combine(Folder, "Original"));

				// Get all the Q&A files.
				string[] files = Directory.GetFiles(Folder, "*.ff");

				foreach (string file in files)
				{
					File.Copy(file, Path.Combine(Path.Combine(Folder, "Original"), Path.GetFileName(file)));
				}

				// Delete the flags file from the backup since it's not a Q&A file.
				if(File.Exists(Path.Combine(Folder, "Original\\flags.ff")))
					File.Delete(Path.Combine(Folder, "Original\\flags.ff"));
			}

			// Save out the memory into encrypted files.
			for(int file = 0; file < 76; file++)
			{
				byte[] crlf = new byte[2];
				crlf[0] = 13; crlf[1] = 10;

				// Get the file name.
				string fileName;
				if (file < 46)
					fileName = "qs" + (file + 1).ToString() + ".ff";
				else
					fileName = "fq" + (file - 45).ToString() + ".ff";

				// Delete the existing file.
				if(File.Exists(Path.Combine(Folder, fileName)))
				{
					File.Delete(Path.Combine(Folder, fileName));
				}

				using(BinaryWriter bw = new BinaryWriter(File.Open(Path.Combine(Folder, fileName), FileMode.OpenOrCreate, FileAccess.Write)))
				{
					for (int question = 0; question < 10; question++)
					{
						// Write out the encoded question.
						byte[] questionBytes = Encode(Files[file].questions[question].question);
						bw.Write(questionBytes);
						bw.Write(crlf);

						// Write out the encoded prefix.
						byte prefixEncoded = (byte)(Files[file].questions[question].prefix + 33);
						bw.Write(prefixEncoded);

						// Determine the number of answers based on the first empty one.
						byte totalAnswers = 0;
						foreach(FFAnswer answer in Files[file].questions[question].answers)
						{
							if(string.IsNullOrEmpty(answer.answer))
							{
								break;
							}
							totalAnswers++;
						}

						// Write total number of answers.
						bw.Write((byte)(totalAnswers + 33));

						// Write out the encoded answers and percentages.
						for(int i = 0; i < totalAnswers; i++)
						{
							// Write out the percentage.
							byte answerPercentageEncoded = Files[file].questions[question].answers[i].percentage;
							answerPercentageEncoded += 33;
							bw.Write(answerPercentageEncoded);

							// Write out the answer.
							byte[] answerBytes = Encode(Files[file].questions[question].answers[i].answer);
							bw.Write(answerBytes);
							bw.Write(crlf);
						}
					}
				}
			}

			MessageBox.Show("New questions saved successfully.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
		}

		private void but_Revert_Click(object sender, EventArgs e)
		{
			// Overwrite the Q&A files with the backup.
			if (MessageBox.Show("Are you sure? This will delete all your current questions.", "Confirm Restore", MessageBoxButtons.YesNo, MessageBoxIcon.Warning) == DialogResult.Yes)
			{
				// Revert to the original game questions.
				if (Directory.Exists(Path.Combine(Folder, "Original")))
				{
					// Get all the Q&A files.
					string[] files = Directory.GetFiles(Path.Combine(Folder, "Original"), "*.ff");

					foreach (string file in files)
					{
						File.Delete(Path.Combine(Folder, Path.GetFileName(file)));
						File.Copy(file, Path.Combine(Folder, Path.GetFileName(file)));
					}

					// Load the original questions.
					LoadQuestions();

					// Force the form to update.
					cbo_Round.SelectedIndex = 0;
					cbo_QuestionBank.SelectedIndex = 0;
					txt_QuestionNo.Text = "1";
					CheckQuestionButtons();
					UpdateForm();

					MessageBox.Show("Original questions and answers have been restored.", "Success", MessageBoxButtons.OK, MessageBoxIcon.Information);
				}
				else
				{
					// Ruh-roh!
					MessageBox.Show("Original backup doesn't exist.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
				}
			}
		}

		private void cbo_QuestionBank_SelectionChangeCommitted(object sender, EventArgs e)
		{
			SaveQuestion();
			txt_QuestionNo.Text = "1";
			CheckQuestionButtons();
			UpdateForm();
		}

		private void cbo_Round_SelectionChangeCommitted(object sender, EventArgs e)
		{
			SaveQuestion();
			UpdateForm();
			LoadQuestionBankCombo();
			txt_QuestionNo.Text = "1";
			CheckQuestionButtons();
		}

		public void LoadQuestionBankCombo()
		{
			// Reset the Question bank combo.
			cbo_QuestionBank.Items.Clear();

			if (cbo_Round.SelectedIndex == 0)
			{
				for (int i = 1; i < 47; i++)
				{
					cbo_QuestionBank.Items.Add("qs" + i.ToString() + ".ff");
				}
			}
			else
			{
				for (int i = 1; i < 31; i++)
				{
					cbo_QuestionBank.Items.Add("fq" + i.ToString() + ".ff");
				}
			}

			cbo_QuestionBank.SelectedIndex = 0;
		}

		public void CheckQuestionButtons()
		{
			if (Convert.ToInt32(txt_QuestionNo.Text) == 1)
				but_Previous.Enabled = false;
			else
				but_Previous.Enabled = true;

			if (Convert.ToInt32(txt_QuestionNo.Text) == 10)
				but_Next.Enabled = false;
			else
				but_Next.Enabled = true;
		}
	}
}
